/*
 * lftp and utils
 *
 * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef CMDEXEC_H
#define CMDEXEC_H

#include <stdarg.h>

#include "Job.h"
#include "ArgV.h"
#include "Filter.h"

#define	CMD(name) Job *CmdExec::do_##name()

class CmdFeeder
{
   CmdFeeder *prev;
public:
   virtual char *NextCmd(class CmdExec *exec,const char *prompt) = 0;
};

class CmdExec : public SessionJob
{
// current command data
   char *cmd;
   ArgV *args;
   FDStream *output;
   bool background;

   char *next_cmd;
   char *cmd_buf;
   bool partial_cmd;

   enum
   {
      COND_ANY,
      COND_AND,
      COND_OR
   }
      condition;

   int	 prev_exit_code;
   int	 exit_code;

   FDStream *source;
   char	 *source_buf;
   int	 source_buf_size;
   char	 *source_buf_ptr;

   void print_cmd_help(const char *cmd);
   void exec_parsed_command();

   enum parse_result
   {
      PARSE_OK,
      PARSE_ERR,
      PARSE_AGAIN
   };
   parse_result parse_one_cmd();

   enum builtins
   {
      BUILTIN_OPEN,
      BUILTIN_CD
   }
      builtin;

   static CmdExec *debug_shell;
   static void debug_callback(char *);

   const char *ResQuery(ResDecl&);

public:
   void FeedCmd(const char *c);
   void PrependCmd(const char *c);
   void ExecParsed(ArgV *a,FDStream *o=0,bool b=false);

   CMD(alias); CMD(anon);  CMD(cd);    CMD(debug);
   CMD(exit);  CMD(get);   CMD(help);  CMD(jobs);
   CMD(kill);  CMD(lcd);   CMD(ls);    CMD(mget);
   CMD(open);  CMD(pwd);   CMD(put);   CMD(set);
   CMD(shell); CMD(source);CMD(user);  CMD(rm);
   CMD(wait);  CMD(site);  CMD(subsh); CMD(mirror);
   CMD(mput);  CMD(mv);	   CMD(cat);   CMD(cache);
   CMD(mkdir); CMD(quote); CMD(scache);CMD(mrm);
   CMD(vers);

   struct cmd_rec
   {
      const char  *name;
      Job   *(CmdExec::*func)();
      const char  *short_desc;
      const char  *long_desc;
   };
   static const cmd_rec cmd_table[];

   static const char * const var_list[];

   CmdExec(FileAccess *s);
   ~CmdExec();

   int Done();
   int ExitCode() { return exit_code; }
   int Do();
   void PrintStatus(int);
   void ShowRunStatus(StatusLine *s);

   char *MakePrompt();

   bool interactive;
   StatusLine *status_line;
   CmdFeeder *feeder;

   static char *command_generator(char *,int);	  // readline completor

   char	 *var_ls;
   char	 *var_prompt;
   bool	 remote_completion;
   int	 long_running;

   void	 Reconfig();

   void	 beep_if_long();
   time_t start_time;

   static CmdExec *cwd_owner;
   char	 *cwd;
   void	 SaveCWD();
   void	 RestoreCWD();

   FDStream *default_output;

   void vfprintf(FILE *file,const char *f,va_list v);
};

#endif CMDEXEC_H
