/*
 * lftp and utils
 *
 * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef JOB_H
#define JOB_H

#include <stdarg.h>
#include <stdio.h>

#include "SMTask.h"
#include "ftpclass.h"
#include "StatusLine.h"

class Job : public SMTask
{
   static void SortJobs();

   Job	 *next;

   static Job *chain;

protected:
   void SetParent(Job *j) { parent=j; }

public:
   int	 jobno;
   Job	 *parent;
   Job	 *waiting;

   void	 AllocJobno();

   virtual void	  PrintStatus(int) {};
   virtual void	  ShowRunStatus(StatusLine *) {}
   virtual void	  SayFinal() {}; // final phrase of fg job
   virtual int	  Done()=0;
   virtual int	  ExitCode() { return 0; }
   virtual int	  Do()=0;

   char	 *cmdline;
   void ListJobs(int verbose_level,int indent=0);
   void ListDoneJobs();
   void BuryDoneJobs();

   static int NumberOfJobs();
   static Job *FindJob(int n);
   static bool Running(int n)
   {
      Job *j=FindJob(n);
      return j && !j->Done();
   }
   void Kill(int n);
   void KillAll();

   virtual void vfprintf(FILE *file,const char *fmt,va_list v);
   void eprintf(const char *fmt,...)
#ifdef __GNUC__
   __attribute__((format(printf,2,3)))
#endif
;
   void fprintf(FILE *file,const char *fmt,...)
#ifdef __GNUC__
   __attribute__((format(printf,3,4)))
#endif
;

   Job();
   virtual ~Job();
};

class SessionPool
{
   static const pool_size=64;
   static FileAccess *pool[pool_size];

public:
   static FileAccess *GetBetter(FileAccess *);
   static void Reuse(FileAccess *);

   static void Print(FILE *f);
   static FileAccess *GetSession(int n);
};

class SessionJob : public Job
{
   static FileAccess *GetBetter(FileAccess *s)
	    { return SessionPool::GetBetter(s); }
   static void Reuse(FileAccess *s) { SessionPool::Reuse(s); }

protected:
   FileAccess *Clone();

   SessionJob(FileAccess *);
   ~SessionJob();

public:
   void	 GetBetter() { session=GetBetter(session); }

   void PrintStatus(int v);
   FileAccess *session;
};

class FtpJob : public SessionJob
{
public:
   FtpJob(Ftp *s) : SessionJob(s) {}
};

#endif /* JOB_H */
