/*
 * lftp and utils
 *
 * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>

#include <errno.h>
#include <unistd.h>
#include <string.h>
#include "QuoteJob.h"

int   QuoteJob::Done()
{
   return session->IsClosed() && in_buf==0;
}

int   QuoteJob::Do()
{
   if(Done())
      return STALL;

   int res;
   if(in_buf>0)
   {
      // try write
      int fd=output->getfd();
      if(fd==-1)
      {
	 if(output->error())
	 {
	    fprintf(stderr,"%s\n",output->error_text);
	 error:
	    session->Close();
	    in_buf=0;
	    delete output;
	    output=0;
	    return MOVED;
	 }
      }
      res=write(fd,buf,in_buf);
      if(res==0)
	 goto error;
      if(res==-1)
      {
	 if(errno!=EAGAIN)
	 {
	    perror(output->name);
	    goto error;
	 }
      }
      else
      {
	 memmove(buf,buf+res,in_buf-=res);
      	 return MOVED;
      }
   }

   if(in_buf==sizeof(buf))
   {
      session->Suspend();
      return STALL;
   }

   // try read
   res=session->Read(buf+in_buf,sizeof(buf)-in_buf);
   if(res==Ftp::DO_AGAIN)
      return STALL;
   if(res<0)
   {
      fprintf(stderr,"%s\n",session->StrError(res));
      output->remove_if_empty();
      goto error;
   }
   if(res==0)
   {
      // EOF
      session->Close();
      return MOVED;
   }
   in_buf+=res;
   return MOVED;
}
